﻿namespace ProjectAirPlane;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningAirPlane
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityAirPlane? EntityAirPlane { get; private set; }

    /// <summary>
    /// Ширина 
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота 
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки самолёта
    /// </summary>
    private int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки самолёта
    /// </summary>
    private int? _startPosY;

    /// <summary>
    /// Ширина прорисовки самолёта
    /// </summary>
    private readonly int _drawningAirPlaneWidth = 197;

    /// <summary>
    /// Высота прорисовки самолёта
    /// </summary>
    private readonly int _drawningAirPlaneHeight = 65;

    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="radar">Признак наличия обвеса</param>
    /// <param name="dopBak">Признак наличия доп бака</param>
    /// <param name="chassi">Признак наличия шасси</param>
    public void Init(int speed, double weight, Color bodyColor, Color additionalColor, bool radar, bool dopBak, bool chassi)
    {
        EntityAirPlane = new EntityAirPlane();
        EntityAirPlane.Init(speed, weight, bodyColor, additionalColor, radar, dopBak, chassi);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        if (_drawningAirPlaneWidth > width || _drawningAirPlaneHeight > height)
        {
            return false;
        }

        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX.HasValue || _startPosY.HasValue)
        {
            if (_startPosX + _drawningAirPlaneWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawningAirPlaneWidth;
            }
            else if (_startPosX < 0) _startPosX = 0;
            if (_startPosY + _drawningAirPlaneHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawningAirPlaneHeight;
            }
            else if (_startPosY < 0) _startPosY = 0;
        }
        return true;

    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы

        if (x + _drawningAirPlaneWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningAirPlaneWidth;
        }
        else if (x < 0) _startPosX = 0;
        else _startPosX = x;

        if (x + _drawningAirPlaneHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningAirPlaneHeight;
        }
        else if (y < 0) _startPosX = 0;
        _startPosY = y;

    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityAirPlane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityAirPlane.Step > 0)
                {
                    _startPosX -= (int)EntityAirPlane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityAirPlane.Step > 0)
                {
                    _startPosY -= (int)EntityAirPlane.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                //TODO прописать логику сдвига вправо
                if (_startPosX + EntityAirPlane.Step + _drawningAirPlaneWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityAirPlane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                //TODO прописать логику сдвига ввниз
                if (_startPosY + EntityAirPlane.Step + _drawningAirPlaneHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityAirPlane.Step;
                }
                return true;
            default:
                return false;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityAirPlane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityAirPlane.AdditionalColor);

        // корпус
        Brush br = new SolidBrush(EntityAirPlane.BodyColor);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 25, 170, 30);
        g.FillRectangle(br, _startPosX.Value, _startPosY.Value + 25, 170, 30);
        // бак
        Brush blackBrush = new SolidBrush(Color.Black);
        g.FillEllipse(blackBrush, _startPosX.Value + 75, _startPosY.Value + 30, 50, 10);
        // нос самолёта
        Pen blackPen = new Pen(Color.Black, 2);
        Brush blueBrush = new SolidBrush(Color.LightBlue);
        Point point1 = new Point(_startPosX.Value + 170, _startPosY.Value + 25);
        Point point2 = new Point(_startPosX.Value + 200, _startPosY.Value + 40);
        Point point3 = new Point(_startPosX.Value + 170, _startPosY.Value + 55);
        Point[] curvePoints =
         {
                 point1,
                 point2,
                 point3,
             };
        g.FillPolygon(blueBrush, curvePoints);
        g.DrawPolygon(blackPen, curvePoints);
        g.DrawLine(blackPen, _startPosX.Value + 170, _startPosY.Value + 40, _startPosX.Value + 200, _startPosY.Value + 40);
        // хвост
        Brush bw = new SolidBrush(EntityAirPlane.BodyColor);
        g.FillPolygon(bw, new Point[]
        {
            new Point((int) _startPosX, (int) _startPosY + 40 ), new Point((int) _startPosX + 80, (int) _startPosY + 30),
            new Point((int) _startPosX, (int) _startPosY), new Point((int) _startPosX ,  (int) _startPosY + 40),



        }
        );
        // шасси
        g.DrawLine(blackPen, _startPosX.Value + 50, _startPosY.Value + 55, _startPosX.Value + 50, _startPosY.Value + 70);
        g.DrawLine(blackPen, _startPosX.Value + 150, _startPosY.Value + 55, _startPosX.Value + 150, _startPosY.Value + 70);
        g.FillEllipse(blackBrush, _startPosX.Value + 40, _startPosY.Value + 65, 10, 10);
        g.FillEllipse(blackBrush, _startPosX.Value + 50, _startPosY.Value + 65, 10, 10);
        g.FillEllipse(blackBrush, _startPosX.Value + 145, _startPosY.Value + 65, 10, 10);
        if (EntityAirPlane.DopBak)
        {
            //бак
            g.FillEllipse(additionalBrush, _startPosX.Value, _startPosY.Value + 45, 40, 20);
            g.DrawEllipse(blackPen, _startPosX.Value, _startPosY.Value + 45, 40, 20);
        }
        if (EntityAirPlane.Radar)
        {
            //радар
            g.DrawLine(blackPen, _startPosX.Value + 60, _startPosY.Value + 25, _startPosX.Value + 60, _startPosY.Value + 15);
            g.DrawLine(blackPen, _startPosX.Value + 60, _startPosY.Value + 15, _startPosX.Value + 67, _startPosY.Value + 11);
            Point point7 = new Point(_startPosX.Value + 60, _startPosY.Value + 15);
            Point point8 = new Point(_startPosX.Value + 60, _startPosY.Value + 5);
            Point point9 = new Point(_startPosX.Value + 70, _startPosY.Value + 25);
            Point[] curvePoints3 =
             {
                 point7,
                 point8,
                 point9,
             };
            g.FillPolygon(additionalBrush, curvePoints3);


        }
    }
}